#include <QtGui>

#include "ogrewidget.h"

class Coordinate3DModifier : public QWidget
{
Q_OBJECT

public:
    Coordinate3DModifier(QWidget *parent=0)
    :QWidget(parent)
    {
        QLabel *lx = new QLabel("X:");
        sbx = new QDoubleSpinBox;
        sbx->setRange(-500, 500);
        
        QLabel *ly = new QLabel("Y:");
        sby = new QDoubleSpinBox;
        sby->setRange(-500, 500);
        
        QLabel *lz = new QLabel("Z:");
        sbz = new QDoubleSpinBox;
        sbz->setRange(-500, 500);
        
        QHBoxLayout *hblayout = new QHBoxLayout;
        hblayout->addWidget(lx);
        hblayout->addWidget(sbx);
        hblayout->addWidget(ly);
        hblayout->addWidget(sby);
        hblayout->addWidget(lz);
        hblayout->addWidget(sbz);
        hblayout->addStretch();
        
        setLayout(hblayout);
        
        connect(sbx, SIGNAL(valueChanged(double)), this, SLOT(onCoordinateChanged()));
        connect(sby, SIGNAL(valueChanged(double)), this, SLOT(onCoordinateChanged()));
        connect(sbz, SIGNAL(valueChanged(double)), this, SLOT(onCoordinateChanged()));
    }
    
public slots:
    void setNewCoordinate(const Ogre::Vector3 &coordinate)
    {
        blockSignals(true);
        sbx->setValue(coordinate.x);
        sby->setValue(coordinate.y);
        sbz->setValue(coordinate.z);
        blockSignals(false);
    }

signals:
    void coordinateChanged(const Ogre::Vector3 &coordinate);

private slots:
    void onCoordinateChanged()
    {
        Ogre::Vector3 newCoord(sbx->value(),
                               sby->value(),
                               sbz->value());
        emit coordinateChanged(newCoord);
    }

private:
    QDoubleSpinBox *sbx, *sby, *sbz;
};

class MainWindow : public QMainWindow
{
Q_OBJECT

public:
    MainWindow()
    :ogreWidget(0)
    {
        ogreWidget = new OgreWidget;
        
        camPosModifier = new Coordinate3DModifier;
        connect(camPosModifier, SIGNAL(coordinateChanged(const Ogre::Vector3&)),
                ogreWidget, SLOT(setCameraPosition(const Ogre::Vector3&)));
        connect(ogreWidget, SIGNAL(cameraPositionChanged(const Ogre::Vector3&)),
                camPosModifier, SLOT(setNewCoordinate(const Ogre::Vector3&)));
        
        createActionMenus();
        createDockWidget();
        
        setCentralWidget(ogreWidget);
    }

private:
    void createActionMenus()
    {
        QAction *closeAct = new QAction("Quitter", this);
        connect(closeAct, SIGNAL(triggered()), this, SLOT(close()));
        
        QAction *changeColorAct = new QAction("Changer la couleur de fond", this);
        connect(changeColorAct, SIGNAL(triggered()), this, SLOT(chooseBgColor()));
        
        QMenu *menu = menuBar()->addMenu("Divers");
        menu->addAction(changeColorAct);
        menu->addAction(closeAct);
    }
    
    void createDockWidget()
    {
        QDockWidget *camPosDock = new QDockWidget("Camera Position", this);
        camPosDock->setAllowedAreas(Qt::TopDockWidgetArea |
                                     Qt::BottomDockWidgetArea);
        camPosDock->setFeatures(QDockWidget::DockWidgetMovable |
                                QDockWidget::DockWidgetFloatable);
        camPosDock->setWidget(camPosModifier);
        
        addDockWidget(Qt::TopDockWidgetArea, camPosDock);
    }
    
private slots:
    void chooseBgColor()
    {
        QColor c = QColorDialog::getColor();
        ogreWidget->setBackgroundColor(c);
    }

private:
    OgreWidget *ogreWidget;
    Coordinate3DModifier *camPosModifier;
};

#include "main.moc"

int main(int argc, char **argv)
{
    QApplication app(argc, argv);
    MainWindow window;
    window.show();
    return app.exec();
}
